@@ -20,8 +20,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#ifndef __lxc_state_h
-#define __lxc_state_h
+#ifndef __LXC_START_H
+#define __LXC_START_H
 
 #include <signal.h>
 #include <sys/param.h>
@@ -72,6 +72,7 @@ struct lxc_handler {
 	int pinfd;
 	const char *lxcpath;
 	void *cgroup_data;
+	int nsfd[LXC_NS_MAX];
 };
 
 extern struct lxc_handler *lxc_init(const char *name, struct lxc_conf *, const char *);
